/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_info.c
 *
 *   Copyright (C) 2000-2003 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_info.c,v 1.8 2003/08/11 00:46:49 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

static struct Message _msgtab[] = {
    {MSG_INFO, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_info, m_info, m_info, m_ignore}
};

static struct Message ic_msgtab[] = {
    {MSG_INFOCONF, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_permission, m_infoconf, m_infoconf, m_ignore}
};

static struct Message c_msgtab[] = {
    {MSG_COPYRIGHT, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_copyright, m_copyright, m_ignore, m_ignore}
};

static char *token = TOK1_INFO;
static char *ic_token = TOK1_INFOCONF;

#ifndef STATIC_MODULES

char *_version = "$Revision: 1.8 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
    mod_add_cmd(ic_msgtab);
    tok1_msgtab[(u_char) *ic_token].msg = ic_msgtab;
    mod_add_cmd(c_msgtab);
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
    mod_del_cmd(ic_msgtab);
    tok1_msgtab[(u_char) *ic_token].msg = NULL;
    mod_del_cmd(c_msgtab);
}
#else
void m_info_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
    mod_add_cmd(ic_msgtab);
    tok1_msgtab[(u_char) *ic_token].msg = ic_msgtab;
    mod_add_cmd(c_msgtab);
}
#endif

/*
 * * m_info 
 *      parv[0] = sender prefix 
 *      parv[1] = servername
 */
int m_info(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char **text;

    static time_t last_used = 0L;

    if (hunt_server(cptr, sptr, ":%s %s :%s", TOK1_INFO, 1, parc, parv) == HUNTED_ISME) {

	text = infotext;

	sendto_lev(SPY_LEV, "INFO requested by %^C [%s]", sptr, sptr->user->server);

	if (!IsAnOper(sptr)) {
	    if (!MyConnect(sptr))
		return 0;
	    if ((last_used + ServerOpts.motd_wait) > NOW)
		return 0;
	    else
		last_used = NOW;
	}
	if (MyConnect(sptr))
	    while (*text)
		send_me_numeric(sptr, RPL_INFO, *text++);

	send_me_numeric(sptr, RPL_INFO, "");

	sendto_one(sptr,
		   ":%C %N %s :Birth Date: %s, compile # %s",
		   &me, RPL_INFO, parv[0], creation, generation);
	sendto_one(sptr, ":%C %N %s :On-line since %s",
		   &me, RPL_INFO, parv[0], smalldate(me.firsttime));
	send_me_numeric(sptr, RPL_ENDOFINFO);

    }

    return 0;
}

/*
 * The whole idea in the following code has been taken from
 * jdcs implementation of the InfoStruct from ircd-hybrid-7
 * -TimeMr14C
 */

static struct ConfigElement serverinfo_ce[] = {
    {"name", CE_STRING, &ServerInfo.name, "Server Name"},
    {"aliasname", CE_STRING, &ServerInfo.aliasname, "Server Name Aliases"},
    {"description", CE_STRING, &ServerInfo.description, "Server Description"},
    {"networkname", CE_STRING, &ServerInfo.networkname, "Network Name"},
    {"networkdesc", CE_STRING, &ServerInfo.networkdesc, "Network Description"},
    {"identity", CE_DECIMAL, &ServerInfo.identity, "Server ID"},
    {NULL},
};

static struct ConfigElement serverhide_ce[] = {
    {"enable", CE_BOOLEAN, &ServerHide.enable, "Server Hide Feature"},
    {"flatten_links", CE_BOOLEAN_YN, &ServerHide.flatten_links, "Flatten Links to one leaf"},
    {"links_from_file", CE_BOOLEAN_YN, &ServerHide.links_from_file, "Read links from a file"},
    {"links_oper_only", CE_BOOLEAN_YN, &ServerHide.links_oper_only, "Only Operators can use links"},
    {NULL},
};

static struct ConfigElement serveropts_ce[] = {
    {"ts_warn_delta", CE_DATE, &ServerOpts.ts_warn_delta, "TS Warning Difference"},
    {"ts_max_delta", CE_DATE, &ServerOpts.ts_max_delta, "TS Maximum Difference"},
    {"save_maxclient_stats", CE_BOOLEAN_YN, &ServerOpts.save_maxclient_stats,
     "Save maximum number of clients"},
    {"use_registerfilter", CE_BOOLEAN, &ServerOpts.use_registerfilter,
     "Enforce +R mode as default"},
    {"use_short_motd", CE_BOOLEAN, &ServerOpts.short_motd, "Use short motd file on connect"},
    {"hide_gcos_with_sgline", CE_BOOLEAN_YN, &ServerOpts.hide_gcos_with_sgline,
     "Replace realname fields"},
    {"realname_replacement", CE_STRING, &ServerOpts.realname_replacement,
     "The string to replace realnames"},
    {"enable_fakehost_mode", CE_BOOLEAN_YN, &ServerOpts.enable_fakehost_mode,
     "enable the usage of umode +x"},
    {"default_fakehost_mode", CE_BOOLEAN_YN, &ServerOpts.default_fakehost_mode,
     "set umode +x on connect"},
    {"use_regex", CE_BOOLEAN_YN, &ServerOpts.use_regex, "use regular expressions"},
    {"usermode_nocolor", CE_BOOLEAN_YN, &ServerOpts.usermode_nocolor, "use +c mode"},
    {"usermode_noctcp", CE_BOOLEAN_YN, &ServerOpts.usermode_noctcp, "use +C mode"},
    {"usermode_hidechans", CE_BOOLEAN_YN, &ServerOpts.usermode_hidechans, "use +w mode"},
    {"usermode_hideidle", CE_BOOLEAN_YN, &ServerOpts.usermode_hideidle, "use +p mode"},
    {"usermode_noprivate", CE_BOOLEAN_YN, &ServerOpts.usermode_noprivate, "use +P mode"},
    {"enable_greeklish", CE_BOOLEAN_YN, &ServerOpts.enable_greeklish, "use +t mode (translation to greeklish)"},
    {NULL},
};

static struct ConfigElement connect_control_ce[] = {
    {"default_kline_time", CE_DATE, &ServerOpts.default_kline_time,
     "Default duration used in klines"},
    {"server_kline_address", CE_STRING, &ServerOpts.server_kline_address, "Server Kline Address"},
    {"network_kline_address", CE_STRING, &ServerOpts.network_kline_address,
     "Network Kline Address"},
    {"wingate_notice", CE_BOOLEAN, &ServerOpts.wingate_notice, "Show a notice about proxy scans"},
    {"monitor_host", CE_STRING, &ServerOpts.monitor_host, "Address where the scans origin"},
    {"monitor_url", CE_STRING, &ServerOpts.proxy_url, "URL where proxy information is shown"},
    {"identd_use_tilde", CE_BOOLEAN_YN, &ServerOpts.identd_use_tilde,
     "Use the ~ sign for users without identd"},
    {"identd_timeout", CE_DATE, &ServerOpts.identd_timeout, "Timeout for an identd lookup"},
    {"identd_complain", CE_BOOLEAN, &ServerOpts.identd_complain, "Complain if identd not found"},
    {NULL},
};

static struct ConfigElement flood_control_ce[] = {
    {"motd_wait", CE_DATE, &ServerOpts.motd_wait, "Time to wait before the next /motd response"},
    {"motd_max", CE_DECIMAL, &ServerOpts.motd_max, "Max number of /motd requests to response"},
    {"client_flood", CE_DECIMAL, &ServerOpts.client_flood,
     "Max number of bytes allowed before flood"},
    {"no_oper_flood", CE_BOOLEAN_YN, &ServerOpts.no_oper_flood,
     "Operators are excluded from flood controls"},
    {"anti_nick_flood", CE_BOOLEAN, &ServerOpts.anti_nick_flood, "Prevent /nick flooding?"},
    {"max_nick_time", CE_DATE, &ServerOpts.max_nick_time,
     "Time to wait before the next /nick response"},
    {"max_nick_changes", CE_DECIMAL, &ServerOpts.max_nick_changes, "Max number of /nick requests"},
    {"no_away_flood", CE_BOOLEAN, &ServerOpts.no_away_flood, "Prevent /away flooding?"},
    {"max_away_time", CE_DATE, &ServerOpts.max_away_time,
     "Time to wait before the next /away response"},
    {"max_away_count", CE_DECIMAL, &ServerOpts.max_away_count, "Max number of /away requests"},
    {"anti_spambot", CE_BOOLEAN, &ServerOpts.anti_spambot, "Prevent joinpart flooding?"},
    {"min_join_leave_time", CE_DATE, &ServerOpts.min_join_leave_time,
     "Time to wait before next joinpart response"},
    {"max_join_leave_count", CE_DECIMAL, &ServerOpts.max_join_leave_count,
     "Max number of joinpart requests"},
    {"join_leave_count_expire_time", CE_DATE, &ServerOpts.join_leave_count_expire_time,
     "Expiration of the joinpart counter"},
    {"oper_spam_countdown", CE_DECIMAL, &ServerOpts.oper_spam_countdown,
     "Number of joinparts before a notice to &SPAMBOTS is sent"},
    {NULL},
};

static struct ConfigElement operconf_ce[] = {
    {"use_remoteconnect", CE_BOOLEAN, &ServerOpts.use_remoteconnect,
     "Show remote connections in &CONNECTS"},
    {"dont_check_remote_qline", CE_BOOLEAN_YN, &ServerOpts.dont_check_qline_remote,
     "Do not check remote clients for qlines"},
    {"staff_fakehost", CE_STRING, &ServerOpts.staffhost, "Default operator fakehost"},
    {"no_oper_jupiter", CE_BOOLEAN, &ServerOpts.no_oper_jupiter,
     "Disallow operators from using /jupiter"},
    {NULL},
};

static struct ConfigElement services_ce[] = {
    {"services_name", CE_STRING, &ServicesConf.services_name, "Services server name"},
    {"nickserv_name", CE_STRING, &ServicesConf.nickserv, "NickServ client name"},
    {"chanserv_name", CE_STRING, &ServicesConf.chanserv, "ChanServ client name"},
    {"memoserv_name", CE_STRING, &ServicesConf.memoserv, "MemoServ client name"},
    {"operserv_name", CE_STRING, &ServicesConf.operserv, "OperServ client name"},
    {"statserv_name", CE_STRING, &ServicesConf.statserv, "StatServ client name"},
    {"helpserv_name", CE_STRING, &ServicesConf.helpserv, "HelpServ client name"},
    {"hide_ulined_servers", CE_BOOLEAN_YN, &ServicesConf.hide_ulined_servers,
     "Hide ULined clients from several outputs"},
    {NULL},
};

static struct ConfigElement channel_ce[] = {
    {"moderaded_hosts", CE_BOOLEAN, &ChannelConf.modhost, "Enable +M moderated hosts mode"},
    {"channelname_bans", CE_BOOLEAN, &ChannelConf.chanban, "Enable +z zapped channels mode"},
    {"halfops", CE_BOOLEAN, &ChannelConf.halfop, "Enable +h halfops mode"},
    {"floodmode", CE_BOOLEAN, &ChannelConf.floodmode, "Enable +f flood protection mode"},
    {"channel_links", CE_BOOLEAN, &ChannelConf.chanlinks, "Enable +L channel redirection mode"},
    {"max_channels_per_user", CE_DECIMAL, &ChannelConf.max_channels_per_user,
     "Maximum number of channels a user allowed to join"},
    {"default_quit_message", CE_STRING, &ChannelConf.default_quit_msg, "Quit message replacement"},
    {"default_extopic_mode", CE_BOOLEAN, &ChannelConf.default_extended_topic_limitation, "Enable default +Tn mode in channels"},
    {NULL},
};

static struct InfoConf ic[] = {
    {"SERVERINFO", serverinfo_ce},
    {"SERVERHIDE", serverhide_ce},
    {"SERVEROPTS", serveropts_ce},
    {"CONNECTCONTROL", connect_control_ce},
    {"FLOODCONTROL", flood_control_ce},
    {"OPERCONF", operconf_ce},
    {"SERVICESCONF", services_ce},
    {"CHANNELS", channel_ce},
    {NULL, NULL},
};

static void parse_infoconf(aClient *cptr, char *type)
{
    int i = 0, j = 0;

    if (irc_strcmp(type, "list") == 0) {
	send_me_numeric(cptr, RPL_INFO, "Following Options are available");
	for (i = 0; ic[i].name; i++)
	    send_me_numeric(cptr, RPL_INFO, ic[i].name);
	return;
    }

    send_me_numeric(cptr, RPL_INFO, "Begin Configuration information");

    for (i = 0; ic[i].name; i++) {
	if (irc_strcmp(ic[i].name, type) == 0) {
	    for (j = 0; ic[i].ce[j].lexer_string; j++) {
		switch (ic[i].ce[j].flags) {
		    case CE_STRING:{
    char *option = *((char **) ic[i].ce[j].element);
			    send_me_debug(cptr, "[Option: %-30s] [Value: %-5s] [%-30s]",
					  ic[i].ce[j].lexer_string,
					  option ? option : "<NONE>", ic[i].ce[j].desc);
			    break;
			}
		    case CE_DECIMAL:{
    int option = *((int *) ic[i].ce[j].element);
			    send_me_debug(cptr, "[Option: %-30s] [Value: %-5d] [%-30s]",
					  ic[i].ce[j].lexer_string, option, ic[i].ce[j].desc);
			    break;
			}
		    case CE_BOOLEAN:{
    int option = *((int *) ic[i].ce[j].element);
			    send_me_debug(cptr, "[Option: %-30s] [Value: %-5s] [%-30s]",
					  ic[i].ce[j].lexer_string,
					  option ? "<ON>" : "<OFF>", ic[i].ce[j].desc);
			    break;
			}
		    case CE_BOOLEAN_YN:{
    int option = *((int *) ic[i].ce[j].element);
			    send_me_debug(cptr, "[Option: %-30s] [Value: %-5s] [%-30s]",
					  ic[i].ce[j].lexer_string,
					  option ? "<YES>" : "<NO>", ic[i].ce[j].desc);
			    break;
			}
		    case CE_DATE:{
    int option = *((int *) ic[i].ce[j].element);
			    send_me_debug(cptr, "[Option: %-30s] [Value: %-5d seconds] [%-30s]",
					  ic[i].ce[j].lexer_string, option, ic[i].ce[j].desc);
			    break;
			}
		    default:{
			    break;
			}
		}		/* switch */
	    }			/* 2nd for */
	}			/* if irc_strcmp */
    }				/* 1st for */

    send_me_numeric(cptr, RPL_INFO, "End Configuration Information");

    return;
}

int m_infoconf(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    if (parc < 2)
	return 0;

    if (hunt_server(cptr, sptr, ":%s %s %s :%s", TOK1_INFOCONF, 2, parc, parv) == HUNTED_ISME) {
	sendto_lev(SPY_LEV, "INFOCONF requested by %^C [%s]", sptr, sptr->user->server);
	parse_infoconf(sptr, parv[1]);
    }

    return 0;
}

int m_copyright(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char **text = copyrighttext;

    static time_t last_used = 0L;

    if (!MyConnect(sptr)) {
	send_me_numeric(sptr, RPL_ENDOFINFO);
	return 0;
    }

    sendto_lev(SPY_LEV, "COPYRIGHT requested by %^C", sptr);

    if (!IsAnOper(sptr)) {
	if ((last_used + ServerOpts.motd_wait) > NOW)
	    return 0;
	else
	    last_used = NOW;
    }

    while (*text)
	send_me_numeric(sptr, RPL_INFO, *text++);

    send_me_numeric(sptr, RPL_INFO, "");

    send_me_numeric(sptr, RPL_ENDOFINFO);

    return 0;
}
